/*
BiConnect4
by Cory McKay

CLASS NAME:		BiConnect4
DESCRIPTION:	A game of Connect4 where pieces may be dropped from the top or right of the board.
				The first player to achieve 4 consecutive pieces wins.
				The methods in this class run the interface of the game and administrate its mechanics.
*/

class BiConnect4
{
	public static void main (String args[])
	// User control
	{
		char[][] board = new char[7][7];	// representation of state
		int turn = 1;						// the current turn number
		Analyzer analyzer = new Analyzer();	// analyzes state space
		char winner = 'c';					// holds w if white is the winner, b if black is the winer
		char player = 'c';					// holds w if user playing white, b if playing black
		char computer = 'c';				// holds w if computer playing white, b if playing black
		boolean valid_input = false;		// indicates whether user has given valid input

		// Initialize board to empty
		for (int i = 0; i < 7; i++)
		{
			for (int j = 0; j < 7; j++)
				board[i][j] = ',';
		}

		// Determine who plays white and who plays black
		valid_input = false;
		while (!valid_input)
		{
			System.out.println("Do you wish to play <w>hite or <black>?");
			player = get_user_input();
			if (player == 'w' || player == 'b') valid_input = true;
			else System.out.println("Invalid Input. Please reenter your selection.");
		}
		if (player == 'w') computer = 'b';
		else computer = 'w';

		// Go through the game turn by turn
		while ((winner = analyzer.is_game_over(board)) == 'c')
		{
			// Output current state
			System.out.println("\nTurn Number: " + turn + "\n");
			for (int i = 0; i < 7; i++)
				System.out.print(i);
			System.out.print("\n");
			for (int i = 0; i < 7; i++)
				System.out.print("|");
			for (int i = 0; i < 7; i++)
			{
				System.out.print("\n");
				for (int j = 0; j < 7; j++)
					System.out.print(board[j][i]);
				System.out.print("-" + ((char) (97  + i))); 
			}
			System.out.println();

			// Process user's turn
			if (is_users_turn(player, turn))
			{
				char move = 'x'; // the user's move
				
				// Get user's input
				valid_input = false;
				while (!valid_input)
				{
					System.out.print("\nPlease enter the row or column of your move: ");
					move = get_user_input();
					if (move == '0' || move == '1' || move == '2' || move == '3' || move == '4' ||
					    move == '5' || move == '6' || move == 'a' || move == 'b' || move == 'c' ||
					    move == 'd' || move == 'e' || move == 'f' || move == 'g') valid_input = true;
					else System.out.print("Invalid Input. Please reenter your selection.");
					System.out.println();
				}
				
				// Update board based on user's move
				analyzer.revise_state(move, board, player);
			}
			
			// Process computer's turn
			else
			{
				char move = analyzer.get_computer_move(board, computer, turn);
				System.out.println("\nComputer chooses move " + move + "\n");
				analyzer.revise_state(move, board, computer);
			}

			// Increment turn
			turn++;
		}

		// Game over
		System.out.println("\n");
		for (int i = 0; i < 7; i++)
			System.out.print(i);
		System.out.print("\n");
		for (int i = 0; i < 7; i++)
			System.out.print("|");
		for (int i = 0; i < 7; i++)
		{
			System.out.print("\n");
			for (int j = 0; j < 7; j++)
				System.out.print(board[j][i]);
			System.out.print("-" + ((char) (97  + i))); 
		}
		System.out.println("\n\n\nGAME OVER");
		if (winner == 't') System.out.println("Tie game");
		else if (winner == player) System.out.println("You win on turn " + turn);
		else System.out.println("Computer wins on turn " + turn);
	}


	private static char get_user_input()
	// Return the last charachter typed by the user before pressing <enter>
	{
		int input_code; // Unicode value for user's response
		char[] input = new char[2]; // holds last two characters entered (including return)
		input[0] = '-';
		input[1] = '-';
		String input_line = new String();
		try
		{
			while ((input_code = System.in.read()) != 10)
			{
				input[0] = input[1];
				input[1] = (char) input_code;
			}
		} catch (Exception e) {}
		return input[0];
	}


	private static boolean is_users_turn(char player, int turn)
	// Return the true if it's the user's turn, false otherwise
	{
		if (turn % 2 == 0 && player == 'b') return true;
		else if (turn % 2 == 1 && player == 'b') return false;
		else if (turn % 2 == 0 && player == 'w') return false;
		else return true;
	}
}